
def funPrettyPlot(x,y, dy_dx):
    yLower1 = min(y)
    yLower2 = min(dy_dx)
    
    yUpper1 = max(y)
    yUpper2 = max(dy_dx)
    
    #Establish upper and lower bounds for y-axis
    yLower = min(yLower1, yLower2)
    yUpper = max(yUpper1, yUpper2)
    
    xLower = min(x)
    xUpper = max(x)
    
    if yLower < 10*xLower:
        yLower = 10*xLower

    if yUpper > 10*xUpper:
            yUpper = 10*xUpper

    from matplotlib import pyplot as plt
    # Define plots.
    plt.figure(figsize=(6, 4)) # Scale
    plt.plot(x, y, label=r'$\operatorname{Re}[f(x + ih)]$')
    plt.plot(x, dy_dx, label=r'$\frac{\operatorname{Im}[f(x + ih)]}{h}$')

# Configure axis properties.
    axes = plt.gca()
    axes.set_xlim([xLower, xUpper])
    axes.set_ylim([yLower, yUpper])

# Configure plot properties.
    plt.xlabel(r'x')
    plt.grid(True)
    plt.legend(loc='upper right')
    plt.tight_layout()
    plt.show()
    
    return

    
def funPlotStepSize(y,h, x0, exactDx, machinePrecision):
    
    import numpy as np
    from matplotlib import pyplot as plt
    
#Implements CTSE method
    def CTSE(fx,x0,h):
     x0 = x0 + h*1j
     y = fx(x0)
     f_x0 = y.real
     dy_dx = y.imag/h
     return dy_dx 
    
    def fd(fx,x0,h):
        dy_dx = (fx(x0 + h) - fx(x0))/h
        return dy_dx
    
    def cd(fx,x0,h):
        dy_dx = (fx(x0 + h) - fx(x0 - h))/(2*h)
        return dy_dx
    

    forwardDifference = []
    centralDifference = []
    ctse = []
    # Compute f'(x) using forward and central difference, and CTSE.
    for i in h:
        nStepSize = 10**-i
        forwardDifference.append(fd(y,x0,nStepSize ))
        centralDifference.append(cd(y,x0,nStepSize ))
        ctse.append(CTSE(y,x0,nStepSize))
    
    # Compute relative errors of the above methods.
    relativeErrorForwardDiff = abs((forwardDifference - exactDx) / exactDx)
    relativeErrorCentralDiff = abs((centralDifference - exactDx) / exactDx)
    relativeErrorCtse = abs((ctse - exactDx) / exactDx)
    
    # Check if a value in list "relativeErrorCtse" is more precise than
    # machine precision, if so, truncate value to machine precision.
    for i, x in enumerate(relativeErrorCtse):
        if x < machinePrecision:
            relativeErrorCtse[i] = machinePrecision
    
    # Plot and configure figures.
    plt.figure(figsize=(7, 4))
    plt.plot(
        h,
        relativeErrorForwardDiff,
        label='Forward Difference',
        marker="^",
        color="blue",
        markerfacecolor="none")
    plt.plot(
        h,
        relativeErrorCentralDiff,
        label='Central Difference',
        marker="s",
        color="purple",
        markerfacecolor="none")
    plt.plot(
        h,
        relativeErrorCtse,
        label='CTSE',
        marker="o",
        color="red",
        markerfacecolor="none")
    axes = plt.gca()
    axes.set_xlim([0, 30])
    axes.set_ylim([10**-17, 10**1])
    plt.xlabel('Step size h')
    plt.ylabel('Relative Error')
    xticksOriginal = [0, 5, 10, 15, 20, 25, 30]
    xticksNew = [
        '$10^{0}$',
        '$10^{-5}$',
        '$10^{-10}$',
        '$10^{-15}$',
        '$10^{-20}$',
        '$10^{-25}$',
        '$10^{-30}$']
    plt.xticks(xticksOriginal, xticksNew)
    plt.grid(True)
    plt.legend()
    plt.tight_layout()
    plt.yscale("log")
    plt.show()
    return